import React from "react";
import { Table } from "@ess-ics/ce-ui-common";
import { Grid, Tooltip } from "@mui/material";
import { noWrapText } from "../common/Helper";
import { useRedirect } from "../../hooks/Redirect";
import { JobStatusIcon, JobTypeIcon } from "./JobIcons";
import { formatDate } from "../common/Helper";

const jobLogColumns = [
  {
    field: "type",
    headerName: "Type",
    flex: 0,
    headerAlign: "center",
    align: "center"
  },
  { field: "ioc", headerName: "IOC name", width: "15ch", sortable: false },
  { field: "version", headerName: "Revision", width: "10ch", sortable: false },
  { field: "host", headerName: "Host", width: "15ch", sortable: false },
  { field: "user", headerName: "User", width: "10ch", sortable: false },
  {
    field: "start",
    headerName: "Time",
    width: "15ch",
    sortable: false,
    headerAlign: "right",
    textAlign: "right"
  }
];

function createGitVersionField(version, shortVersion) {
  return (
    <Tooltip
      title={version}
      arrow
      enterDelay={400}
    >
      <label>{noWrapText(shortVersion ?? version)}</label>
    </Tooltip>
  );
}

function createTableRow(job, colorStyle) {
  return {
    id: job.id,
    type: (
      <Grid
        container
        direction="column"
        justifyContent="center"
        alignItems="center"
      >
        <JobTypeIcon
          type={job.type}
          colorStyle={colorStyle}
        />
      </Grid>
    ),
    status: (
      <Grid
        container
        direction="column"
        justifyContent="center"
        alignItems="center"
      >
        <JobStatusIcon status={job.status} />
      </Grid>
    ),
    ioc: noWrapText(job.iocName),
    version: createGitVersionField(job.gitReference, job.sourceVersionShort),
    host: noWrapText(job.host?.hostName ?? null),
    network: noWrapText(job.host?.network ?? null),
    user: noWrapText(job.createdBy),
    start: noWrapText(formatDate(job.createdAt)),
    deploymentFailed: job.status === "FAILED" || job.status === "ERROR",
    jobType: job.type,
    iocId: job.iocId,
    deploymentId: job.deploymentId
  };
}

function createTableRowJobLog(job) {
  return createTableRow(job, "black");
}

export function JobTable({
  jobs,
  rowType = "jobLog",
  pagination,
  onPage,
  loading
}) {
  const tableTypeSpecifics = {
    jobLog: [jobLogColumns, createTableRowJobLog]
  };

  const redirect = useRedirect();
  const [columns, createRow] = tableTypeSpecifics[rowType];

  const onRowClick = (params) => {
    const job = params.row;
    if (["DEPLOY", "UNDEPLOY", "START", "STOP"].includes(job?.jobType)) {
      redirect(`/jobs/${job?.id}`);
    }
  };

  return (
    <Table
      columns={columns}
      rows={jobs?.map((job) => createRow(job))}
      onRowClick={onRowClick}
      pagination={pagination}
      onPage={onPage}
      loading={loading}
      autoHeight
    />
  );
}
